// Copyright (C) 2001, 2004  EPITA Research and Development Laboratory
//
// This file is part of the Olena Library.  This library is free
// software; you can redistribute it and/or modify it under the terms
// of the GNU General Public License version 2 as published by the
// Free Software Foundation.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this library; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
// Boston, MA 02110-1301, USA.
//
// As a special exception, you may use this file as part of a free
// software library without restriction.  Specifically, if other files
// instantiate templates or use macros or inline functions from this
// file, or you compile this file and link it with other files to
// produce an executable, this file does not by itself cause the
// resulting executable to be covered by the GNU General Public
// License.  This exception does not however invalidate any other
// reasons why the executable file might be covered by the GNU General
// Public License.

#ifndef OLENA_CORE_ACCUM_HH
# define OLENA_CORE_ACCUM_HH


/*! \class max_accumulator
**
** This is a \a functor. It saves the maximum T value
** that has been passed as an argument of its operator()
** method. To retrieve the value saved, just use the
** max_accumulator as a T instance.
*/

template <class T>
struct max_accumulator
{
  
  max_accumulator (T t) : acc_(t) 
  {}

  void 
  operator()(T t)
  {
    if (t > acc_)
      acc_ = t;
  }
  
  operator T() const
  {
    return acc_;
  }

private:
  
  T acc_;
  
};

#endif // OLENA_CORE_ACCUM_HH
